import type { NodeStruct } from "ts-common";
import type { QueryCacheData } from "./model";
import { getRelationCypher } from "./relation";

export function getNodeCypher(JSONQuery: NodeStruct): [string, QueryCacheData] {
    let label = "";
    if (JSONQuery.label) {
        label = `:${JSONQuery.label}`;
    }
    let id = "";
    if (JSONQuery.id) {
        id = JSONQuery.id;
    }

    const cacheData: QueryCacheData = { entities: [], relations: [] };

    if (id) {
        cacheData.entities.push({ id, queryId: "" });
    }

    let cypher = `(${id}${label})`;

    if (JSONQuery.relation) {
        const [relationCypher, subCache] = getRelationCypher(JSONQuery.relation);

        if (relationCypher) {
            cacheData.entities.push(...subCache.entities);
            cacheData.relations.push(...subCache.relations);
            if (JSONQuery.relation.direction === "FROM") {
                cypher += `<-${relationCypher}`;
            } else if (JSONQuery.relation.direction === "TO") {
                cypher += `-${relationCypher}`;
            } else {
                cypher += `-${relationCypher}`;
            }
        }
    }
    return [cypher, cacheData];
}